@@ -5,6 +5,7 @@ Module with several geodetic related methods
 # import math
 # import sys
 import collections
+import math
 import typing
 
 import numpy as np
@@ -18,11 +19,16 @@ WGS84_E = 8.1819190842622e-2
 DMS = collections.namedtuple('DMS', 'degrees minutes seconds')
 LLA = collections.namedtuple('LLA', 'longitude latitude altitude')
 XYZ = collections.namedtuple('XYZ', 'x y z')
+ENU = collections.namedtuple('ENU', 'east north up')
 ElAz = collections.namedtuple('ElAz', 'elevation azimuth')
 
 class Coordinates(object):
+    """
+    Class to store coordinates and transform among various coordinate systems
+    (cartesian and geodetic)
+    """
 
-    def __init__(self, x:float, y:float, z:float) -> None:
+    def __init__(self, x: float, y: float, z: float) -> None:
         self.point = XYZ(x, y, z)
 
     def is_valid(self) -> bool:
@@ -41,7 +47,7 @@ class Coordinates(object):
         """
         return self.point
 
-    def lla(self, radians:bool=False, a:float=WGS84_A, e:float=WGS84_E) -> LLA:
+    def lla(self, radians: bool = False, a: float = WGS84_A, e: float = WGS84_E) -> LLA:
         """
         >>> coordinates = Coordinates(4889803.653, 170755.706, 4078049.851)
         >>> "lon: {:.0f} lat: {:.0f} height: {:.0f}".format(*coordinates.lla())
@@ -49,7 +55,7 @@ class Coordinates(object):
         """
         return LLA(*xyz_to_lla(*self.point, radians=radians, a=a, e=e))
 
-    def dms(self, radians:bool=False, a:float=WGS84_A, e:float=WGS84_E) -> LLA:
+    def dms(self, radians: bool = False, a: float = WGS84_A, e: float = WGS84_E) -> LLA:
         """
         >>> coordinates = Coordinates(4889803.653, 170755.706, 4078049.851)
         >>> coord = coordinates.dms()
@@ -72,8 +78,23 @@ class Coordinates(object):
         """
         return f'{self.point.x:.3f} {self.point.y:.3f} {self.point.z:.3f}'
 
+
+    def __sub__(self, other) -> XYZ:
+        """
+        >>> dest_position = Coordinates(4889804, 170755, 4078049)
+        >>> orig_position = Coordinates(4889803, 170757, 4078049)
+        >>> dest_position - orig_position
+        XYZ(x=1, y=-2, z=0)
+        """
+
+        xyz_self = self.xyz()
+        xyz_other = other.xyz()
+
+        return XYZ(x=xyz_self.x-xyz_other.x, y=xyz_self.y-xyz_other.y, z=xyz_self.z-xyz_other.z)
+
     @staticmethod
-    def from_lla(longitude_deg:float, latitude_deg:float, height_m:float, a:float=WGS84_A, e:float=WGS84_E) -> 'Coordinates':
+    def from_lla(longitude_deg: float, latitude_deg: float, height_m: float,
+                 a: float = WGS84_A, e: float = WGS84_E) -> 'Coordinates':
         """
         >>> coordinates = Coordinates.from_lla(2, 40, 100)
         >>> coordinates.is_valid()
@@ -85,7 +106,7 @@ class Coordinates(object):
         return Coordinates(*xyz)
 
     @staticmethod
-    def from_xyz(x:float, y:float, z:float) -> 'Coordinates':
+    def from_xyz(x: float, y: float, z: float) -> 'Coordinates':
         """
         >>> coordinates = Coordinates.from_lla(4889803.653, 170755.706, 4078049.851)
         >>> coordinates.is_valid()
@@ -94,6 +115,121 @@ class Coordinates(object):
 
         return Coordinates(x, y, z)
 
+# ------------------------------------------------------------------------------
+
+class CoordinatesDisplacement(object):
+    """
+    Representation of a displacement relative to a certain point (Coordinates).
+    Use this class to represent deviations such as error vector, lever arms, ...
+    """
+
+    def __init__(self, xyz: XYZ = None, enu: ENU = None, ref_position: Coordinates = None):
+        """
+        >>> c = CoordinatesDisplacement(xyz=XYZ(0.1, 0.1, 0.1))
+        >>> c.is_valid()
+        True
+        """
+        self.xyz = xyz
+        self.enu = enu
+        self.ref_lla = None
+        
+        if ref_position:
+            self.set_reference_position(ref_position)
+
+    def set_reference_position(self, position: Coordinates) -> None:
+
+        if not position:
+            raise ValueError("Invalid reference position")
+
+        self.ref_lla = position.lla(radians=True)
+
+    def is_valid(self) -> bool:
+        """
+        >>> c = CoordinatesDisplacement()
+        >>> c.is_valid()
+        False
+        """
+        return self.xyz is not None or self.enu is not None
+
+    def check_validity(self) -> None:
+        if not self.is_valid():
+            raise ValueError("CoordinatesDisplacement object is not valid")
+
+    def get_xyz(self) -> XYZ:
+        """
+        >>> enu = ENU(-0.416, 5.664, -0.081)
+        >>> reference_position = Coordinates.from_lla(1.52160871, 41.26531742, 127.0)
+        >>> c = CoordinatesDisplacement(enu=enu)
+        >>> c.set_reference_position(reference_position)
+        >>> xyz = c.get_xyz()
+        >>> '{:.03f} {:.03f} {:.03f}'.format(*xyz)
+        '-3.784 -0.517 4.204'
+        """
+
+        self.check_validity()
+
+        if not self.xyz:
+            if not self.ref_lla:
+                raise ValueError("Missing reference position in CoordinatesDisplacement to convert frame (ENU to XYZ)")
+            xyz_list = enu_to_ecef(self.ref_lla.longitude, self.ref_lla.latitude, *self.enu, radians=True)
+            self.xyz = XYZ(*xyz_list)
+            
+        return self.xyz 
+
+    def get_enu(self) -> ENU:
+        """
+        >>> xyz = XYZ(0.1, 0.1, 0.1)
+        >>> ref_position = Coordinates(4889803.653, 170755.706, 4078049.851)
+        >>> c = CoordinatesDisplacement(xyz=xyz, ref_position=ref_position)
+        >>> enu = c.get_enu()
+        >>> '{:.03f} {:.03f} {:.03f}'.format(*enu)
+        '0.096 0.010 0.144'
+        """
+
+        self.check_validity()
+
+        if not self.enu:
+            if not self.ref_lla:
+                raise ValueError("Missing reference position in CoordinatesDisplacement to convert frame (XYZ to ENU)")
+            enu_list = ecef_to_enu(self.ref_lla.longitude, self.ref_lla.latitude, *self.xyz, radians=True)
+            self.enu = ENU(*enu_list)
+            
+        return self.enu
+
+    def __abs__(self):
+        """
+        >>> c = CoordinatesDisplacement.from_xyz(0.1, 0.1, 0.1)
+        >>> f'{abs(c):.4f}'
+        '0.1732'
+        """
+
+        if not self.is_valid():
+            raise ValueError("Cannot compute the magnitude of an empty array")
+
+        vector = self.xyz or self.enu
+
+        return math.sqrt(sum([v * v for v in vector]))
+
+
+    @staticmethod
+    def from_xyz(x: float, y: float, z: float):
+        """
+        >>> c = CoordinatesDisplacement.from_xyz(0.1, 0.1, 0.1)
+        >>> c.is_valid()
+        True
+        """
+        return CoordinatesDisplacement(xyz=XYZ(x, y, z))
+
+    @staticmethod
+    def from_enu(e: float, n: float, u: float):
+        """
+        >>> c = CoordinatesDisplacement.from_enu(0.1, 0.1, 0.1)
+        >>> c.is_valid()
+        True
+        """
+        return CoordinatesDisplacement(enu=ENU(e, n, u))
+
+# ------------------------------------------------------------------------------
 
 def valid_coordinates(lla=None, xyz=None):
     """
